@@ -12,17 +12,130 @@ use Exporter "import";
 
 use AutoLoader;
 
-use Test::Deep qw<eq_deeply>;
-use Net::IP;#::XS;
+use Net::IP::Lite qw<ip2bin>;
+use Data::Cmp qw<cmp_data>;
+
 use Sort::Key::Multi qw<siikeysort>;
 
-use Scalar::Util qw<looks_like_number>;
+#use Scalar::Util qw<looks_like_number>;
 use Data::Combination;
 
-our @af_2_name;
-our %name_2_af;
-our @sock_2_name;
-our %name_2_sock;
+
+##DIRECT COPY FROM Net::IP
+##########################
+my $ERROR;
+my $ERRNO;
+# Definition of the Ranges for IPv4 IPs
+my %IPv4ranges = (
+    '00000000'                         => 'PRIVATE',     # 0/8
+    '00001010'                         => 'PRIVATE',     # 10/8
+    '0110010001'                       => 'SHARED',      # 100.64/10
+    '01111111'                         => 'LOOPBACK',    # 127.0/8
+    '1010100111111110'                 => 'LINK-LOCAL',  # 169.254/16
+    '101011000001'                     => 'PRIVATE',     # 172.16/12
+    '110000000000000000000000'         => 'RESERVED',    # 192.0.0/24
+    '110000000000000000000010'         => 'TEST-NET',    # 192.0.2/24
+    '110000000101100001100011'         => '6TO4-RELAY',  # 192.88.99.0/24 
+    '1100000010101000'                 => 'PRIVATE',     # 192.168/16
+    '110001100001001'                  => 'RESERVED',    # 198.18/15
+    '110001100011001101100100'         => 'TEST-NET',    # 198.51.100/24
+    '110010110000000001110001'         => 'TEST-NET',    # 203.0.113/24
+    '1110'                             => 'MULTICAST',   # 224/4
+    '1111'                             => 'RESERVED',    # 240/4
+    '11111111111111111111111111111111' => 'BROADCAST',   # 255.255.255.255/32
+);
+ 
+# Definition of the Ranges for Ipv6 IPs
+my %IPv6ranges = (
+    '00000000'                                      => 'RESERVED',                  # ::/8
+    ('0' x 128)                                     => 'UNSPECIFIED',               # ::/128
+    ('0' x 127) . '1'                               => 'LOOPBACK',                  # ::1/128
+    ('0' x  80) . ('1' x 16)                        => 'IPV4MAP',                   # ::FFFF:0:0/96
+    '00000001'                                      => 'RESERVED',                  # 0100::/8
+    '0000000100000000' . ('0' x 48)                 => 'DISCARD',                   # 0100::/64
+    '0000001'                                       => 'RESERVED',                  # 0200::/7
+    '000001'                                        => 'RESERVED',                  # 0400::/6
+    '00001'                                         => 'RESERVED',                  # 0800::/5
+    '0001'                                          => 'RESERVED',                  # 1000::/4
+    '001'                                           => 'GLOBAL-UNICAST',            # 2000::/3
+    '0010000000000001' . ('0' x 16)                 => 'TEREDO',                    # 2001::/32
+    '00100000000000010000000000000010' . ('0' x 16) => 'BMWG',                      # 2001:0002::/48            
+    '00100000000000010000110110111000'              => 'DOCUMENTATION',             # 2001:DB8::/32
+    '0010000000000001000000000001'                  => 'ORCHID',                    # 2001:10::/28
+    '0010000000000010'                              => '6TO4',                      # 2002::/16
+    '010'                                           => 'RESERVED',                  # 4000::/3
+    '011'                                           => 'RESERVED',                  # 6000::/3
+    '100'                                           => 'RESERVED',                  # 8000::/3
+    '101'                                           => 'RESERVED',                  # A000::/3
+    '110'                                           => 'RESERVED',                  # C000::/3
+    '1110'                                          => 'RESERVED',                  # E000::/4
+    '11110'                                         => 'RESERVED',                  # F000::/5
+    '111110'                                        => 'RESERVED',                  # F800::/6
+    '1111110'                                       => 'UNIQUE-LOCAL-UNICAST',      # FC00::/7
+    '111111100'                                     => 'RESERVED',                  # FE00::/9
+    '1111111010'                                    => 'LINK-LOCAL-UNICAST',        # FE80::/10
+    '1111111011'                                    => 'RESERVED',                  # FEC0::/10
+    '11111111'                                      => 'MULTICAST',                 # FF00::/8
+);
+
+#------------------------------------------------------------------------------
+# Subroutine ip_iptypev4
+# Purpose           : Return the type of an IP (Public, Private, Reserved)
+# Params            : IP to test, IP version
+# Returns           : type or undef (invalid)
+sub ip_iptypev4 {
+    my ($ip) = @_;
+    no warnings "uninitialized";
+ 
+    # check ip
+    if ($ip !~ m/^[01]{1,32}$/) {
+        $ERROR = "$ip is not a binary IPv4 address $ip";
+        $ERRNO = 180;
+        return;
+    }
+     
+    # see if IP is listed
+    foreach (sort { length($b) <=> length($a) } keys %IPv4ranges) {
+        return ($IPv4ranges{$_}) if ($ip =~ m/^$_/);
+    }
+ 
+    # not listed means IP is public
+    return 'PUBLIC';
+}
+ 
+#------------------------------------------------------------------------------
+# Subroutine ip_iptypev6
+# Purpose           : Return the type of an IP (Public, Private, Reserved)
+# Params            : IP to test, IP version
+# Returns           : type or undef (invalid)
+sub ip_iptypev6 {
+    my ($ip) = @_;
+    no warnings "uninitialized";
+
+    # check ip
+    if ($ip !~ m/^[01]{1,128}$/) {
+        $ERROR = "$ip is not a binary IPv6 address";
+        $ERRNO = 180;
+        return;
+    }
+     
+    foreach (sort { length($b) <=> length($a) } keys %IPv6ranges) {
+        return ($IPv6ranges{$_}) if ($ip =~ m/^$_/);
+    }
+ 
+    # How did we get here? All IPv6 addresses should match 
+    $ERROR = "Cannot determine type for $ip";
+    $ERRNO = 180;
+    return;
+}
+
+#######
+#END COPY FROM Net::IP
+
+my @af_2_name;
+my %name_2_af;
+my @sock_2_name;
+my %name_2_sock;
 my $IPV4_ANY="0.0.0.0";
 my $IPV6_ANY="::";
 
@@ -43,6 +156,14 @@ BEGIN{
 
 
 	@names=grep /^SOCK_/, keys %Socket::;
+
+	#filter out the following bit masks on BSD, to prevent a huge array:
+	#define	SOCK_CLOEXEC	0x10000000
+	#define	SOCK_NONBLOCK	0x20000000
+	
+	for my $ignore(qw<SOCK_CLOEXEC SOCK_NONBLOCK>){
+		@names=grep $_ ne $ignore, @names;
+	}
 	for my $name (@names){
 		my $val;
 		eval {
@@ -70,6 +191,8 @@ our %EXPORT_TAGS = ( 'all' => [ qw(
 	if_nametoindex
 	if_indextoname
 	if_nameindex
+	has_IPv4_interface
+	has_IPv6_interface
 
 ) ] );
 
@@ -80,7 +203,7 @@ our @EXPORT = qw(
 	
 );
 
-our $VERSION = '0.1.0';
+our $VERSION = '0.2.0';
 
 sub getifaddrs;
 sub AUTOLOAD {
@@ -117,7 +240,9 @@ XSLoader::load('Socket::More', $VERSION);
 
 sub socket {
 
-	return &CORE::socket if looks_like_number $_[1];
+	require Scalar::Util;
+	#qw<looks_like_number>;
+	return &CORE::socket if Scalar::Util::looks_like_number $_[1];
 
 	if(ref($_[1]) eq "HASH"){
 		#assume a 'interface object no need for remaining args
@@ -322,7 +447,8 @@ sub sockaddr_passive{
 				#$interface->{port}=$_->{port};
 				$clone->{interface}=$interface->{name};
 				#$clone->{group}=Net::IP::XS::ip_iptypev4(Net::IP::XS->new($clone->{address})->binip);
-				$clone->{group}=Net::IP->new($clone->{address})->iptype;
+				$clone->{group}=ip_iptypev4 ip2bin($clone->{address});
+				#$clone->{group}=Net::IP->new($clone->{address})->iptype;
 			}
 			elsif($fam == AF_INET6){
 				my(undef, $ip, $scope, $flow_info)=unpack_sockaddr_in6($interface->{addr});
@@ -330,7 +456,10 @@ sub sockaddr_passive{
 				$clone->{address}=inet_ntop($fam, $ip);
 				$clone->{interface}=$interface->{name};
 				#$clone->{group}=Net::IP::XS::ip_iptypev6(Net::IP::XS->new($clone->{address})->binip);
-				$clone->{group}=Net::IP->new($clone->{address})->iptype;
+				
+				$clone->{group}=ip_iptypev6 ip2bin($clone->{address});
+
+				#$clone->{group}=Net::IP->new($clone->{address})->iptype;
 			}
 			elsif($fam == AF_UNIX){
 				my $suffix=$_->{type}==SOCK_STREAM?"_S":"_D";
@@ -365,7 +494,8 @@ sub sockaddr_passive{
         push @list, $output[0] if @output;
         for(my $i=1; $i<@output; $i++){
                 my $out=$output[$i];
-                my $found=List::Util::first {eq_deeply $_, $out} @list;
+		#my $found=List::Util::first {eq_deeply $_, $out} @list;
+                my $found=List::Util::first {cmp_data($_, $out)==0} @list;
                 push @list, $out unless $found;
         }
 
@@ -493,6 +623,30 @@ sub string_to_sock {
 	@name_2_sock{@found};
 }
 
+sub has_IPv4_interface {
+	my $spec={
+		family=>AF_INET,
+		type=>SOCK_STREAM,
+		port=>0
+	};
+	my @results=sockaddr_passive $spec;
+	
+	@results>=1;
+
+}
+
+sub has_IPv6_interface{
+	my $spec={
+		family=>AF_INET6,
+		type=>SOCK_STREAM,
+		port=>0
+	};
+	my @results=sockaddr_passive $spec;
+	
+	@results>=1;
+
+}
+
 1;
 __END__
 
